from pydantic import BaseModel


#用户注册，编辑
class SignUpdateModel(BaseModel):
    username: str
    name: str
    role_id: str
    password: str
    sex: int

#用户登录
class SignInModel(BaseModel):
    username: str
    password: str

#用户分页列表
class UserModel(BaseModel):
    page_no: int = 1
    page_size: int = 5

#下位机发送指令
class SendMesModel(BaseModel):
    name: str
    status:int = 0

#新增试验
class InsertModel(BaseModel):
    items: dict


class ContinueModel(BaseModel):
    id: str
    step: str

class UpdateModel(BaseModel):
    id: str
    items: dict

#停止试验
class StopModel(BaseModel):
    id: str
    test_result: int = 0


class SpecimenSearchModel(BaseModel):
    page_no: int = 1
    page_size: int = 5
    specimen_name: str = None
    specimen_code: str = None
    specimen_id: str = None
    rfid: str = None


class WaitSpecimenSearchModel(BaseModel):
    page_no: int = 1
    page_size: int = 5
    record_id: str = None
    specimen_name: str = None
    specimen_code: str = None
    specimen_id: str = None
    rfid: str = None


#报表查询
class SearchWordModel(BaseModel):
    page_no: int = 1
    page_size: int = 5
    name: str = None
    experiment_id: str = None
    record_id: str = None
    date: str = None

class CombustionLimitModel(BaseModel):
    specimen_id: str
    ct_value: str
    combustion_type: str


#报表导出
class ExportTableModel(BaseModel):
    specimen_id: str
    download_type: str


# 获取实验数据
class DataModel(BaseModel):
    id: str


class SpecimenModel(BaseModel):
    items: dict


class SpecimenConfigModel(BaseModel):
    items: dict


class InStorageModel(BaseModel):
    specimen_ids: list
    storage_type: int


class DeviceConfigModel(BaseModel):
    light_status: int # 0-关闭 1-开启
    fan_status: int # 0-关闭 1-开启
    warning_light_status: int = 0# 0-关闭 1-开启
    lock_status: int # 0-关闭 1-开启


class ExperimentParamModel(BaseModel):
    temperature: float  # 设置温度
    expose_time: float  # 暴露时间
    put_time: float     # 放置时间
    short_circuit_param: float = 0.0     # 短路参数
    specimen_name: str     # 样品名称
    battery_type: str     # 电池型号
    specimen_num: str     # 样品编号
    battery_specification: str     # 电池规格
    battery_manufacturers: str     # 电池厂家
    channel_ids: str     # 通道ids   逗号分隔
    experiment_specimen_id: str = ''     # 材料id
    channel_experiment_dict : dict = {}     # 通道实验参数
